from behave import *
from os import environ
from polibeepsync.common import User
import requests



@when('I select the courses tab')
def step_impl(context):
    assert False

@then('I get all available courses')
def step_impl(context):
    assert False


@when('I press the button "select all"')
def step_impl(context):
    assert False

@then('All courses should be selected')
def step_impl(context):
    assert False

@when('I press the button "unselect all"')
def step_impl(context):
    assert False

@then('No course should be selected')
def step_impl(context):
    assert False

@when('I\'m presented with the course list')
def step_impl(context):
    assert False

@then('all courses are selected except "BeeP Channel"')
def step_impl(context):
    assert False